/* alternatives subsystem
(c) 2003
Stanislav Ievlev <inger@altlinux.org>
*/

#ifndef ALTERNATIVE_H
#define ALTERNATIVE_H

#include <unistd.h>

#include <alternatives/candidate.h>

class AltStorage;
class Alternative;

/**
 * hash of possible alternatives: key - fake name, value - alternative object
 */
typedef std::map<std::string, Alternative *> alternative_list;

/**
 * set of possible candidates, alternative representation
 */
class Alternative
{
	friend class AltStorage;
public:
	/**
	 * A constructor, initialize default values
	 */
	Alternative(const std::string& name);
	/**
	 * A destructor, clear internal structures
	 */
	~Alternative();
	/**
	 * setup/update alternative item
	 * @param other new alternative
	 */
	Alternative &operator<<=(Alternative &other);
	/**
	 * @return current preffered candidate
	 */
	std::string points_to() const;
	/**
	 * @return fake name of the alternative
	 */
	const std::string& name() const;
	/**
	 * @return current candidate in manual mode, empty string in auto mode
	 */
	const std::string& current() const;
	/**
	 * setup current candidate (set alternative into manual mode)
	 */
	void	set_current(const std::string& str);

	candidate_list	candidates; /**< Possible candidates */

	alternative_list	slaves; /**< possible slave alternatives: key - realname, value - alternative */
	Alternative	*parent; /**< pointer to parent alternative if it's slave otherwise zero */

private:
	/**
	 * set _dumped flag, this alternative will be dumped into dump.xml file at the finish of work
	 * @value new value of the flag
	 */
	void set_dumped(bool value = false);
	/**
	 * @return are we need to dump state of this alternative into dump.xml file
	 */
	bool dumped() const;

	bool	_dumped; /**< flag: this is a file from dump? */
	std::string	_name; /**< fake name of the alternative */
	std::string	_current; /**< current candidate name */
};


#endif
